package com.plannaplan.entities;

import java.sql.Timestamp;
import java.util.concurrent.TimeUnit;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;

import com.plannaplan.types.UserRoles;

/**
 * Entity of User grouping of state ssociated about
 * id,name,surname,email,role,token,tokenCreatedDate
 */

@Entity
public class User {
    private static final float TOKEN_EXPIRE_MINUTES = 15;

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;
    private String name;
    private String surname;
    private String email;
    private String usosId;
    private UserRoles role;
    private String token;
    private Timestamp tokenUsageDate;

    public User() {
    }

    /**
     * 
     * @param name    name given to the user
     * @param surname surname given to the user
     * @param mail    mail given to the user
     * @param role
     */
    public User(String name, String surname, String mail, UserRoles role) {
        this.name = name;
        this.surname = surname;
        this.email = mail;
        this.role = role;
    }

    /**
     * 
     * @param name    name given to the user
     * @param surname surname given to the user
     * @param mail    mail given to the user
     * @param usosId  id in the USOS system
     * @param role
     */
    public User(String name, String surname, String mail, String usosId, UserRoles role) {
        this(name, surname, mail, role);
        this.usosId = usosId;
    }

    /**
     * usos id getter
     * 
     * @return usosid
     */
    public String getUsosId() {
        return usosId;
    }

    /**
     * email getter
     * 
     * @return mailof user
     */
    public String getEmail() {
        return email;
    }

    /**
     * email setter
     * 
     * @param email user email
     */
    public void setEmail(String email) {
        this.email = email;
    }

    /**
     * token usage getter
     * 
     * @return Timestamp when token was used
     */
    public Timestamp getTokenUsageDate() {
        return tokenUsageDate;
    }

    /**
     * token getter
     * 
     * @return user token
     */
    public String getToken() {
        return token;
    }

    /**
     * token seter. Sets token and automaticly set time when was set
     * 
     * @param token token to set
     */
    public void setToken(String token) {
        this.tokenUsageDate = new Timestamp(System.currentTimeMillis());
        this.token = token;
    }

    /**
     * name setter
     * 
     * @return String user name
     */
    public String getName() {
        return name;
    }

    /**
     * user rolse getter
     * 
     * @return UserRoles of user
     */
    public UserRoles getRole() {
        return role;
    }

    /**
     * user role setter
     * 
     * @param role to be set
     */
    public void setRole(UserRoles role) {
        this.role = role;
    }

    /**
     * surname getter
     * 
     * @return string surname
     */
    public String getSurname() {
        return surname;
    }

    /**
     * surname setter
     * 
     * @param surname string to be set as surnames
     */
    public void setSurname(String surname) {
        this.surname = surname;
    }

    /**
     * name stter
     * 
     * @param name stirng to be set as name
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * id getter
     * 
     * @return id in database
     */
    public Long getId() {
        return this.id;
    }

    /**
     * it checks if given ammount of time passed since last token usage. If not
     * retunr true and reset time otherwise return false and token won work anymore
     * 
     * @return boolena if credentials (token) is expired or not
     */
    public boolean isCredentialsNonExpired() {
        final long diffInMilliseconds = Math
                .abs(this.tokenUsageDate.getTime() - new Timestamp(System.currentTimeMillis()).getTime());
        final long minutes = TimeUnit.MILLISECONDS.toMinutes(diffInMilliseconds);

        if (minutes > TOKEN_EXPIRE_MINUTES) {
            return false;
        }
        this.tokenUsageDate = new Timestamp(System.currentTimeMillis());
        return true;
    }
}
